/*
   Метеостанция на ESP8266 от it4it.club
   https://it4it.club/topic/55-meteostantsiya-na-esp8266-ot-it4itclub/

   Задействованные пины
   ---------------------------------------
   NodeMCU Pins | ESP 07/12 Pins | Датчики
   ---------------------------------------
        D2      |      GPIO 4    |   SDA
        D1      |      GPIO 5    |   SCL
        3.3V    |      VCC       |   VCC
        GND     |      GND       |   GND

   Учтите, что если Вы раскомментируйте несколько библиотек, использующих датчики на одном адресе I2C шины, 
   то ни к чему хорошему это не приведет. Один адрес = один датчик = одна библиотека. 
   Допустимо использование однотипных датчиков, но только с разными адресами и Вам придется самостоятельно 
   добавить необходимую логику для второго датчика.
*/
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <ESP8266HTTPClient.h>
#include <ESP8266WebServer.h>
#include <WiFiClient.h>
#include <PubSubClient.h> // https://github.com/knolleary/pubsubclient
#include <DNSServer.h>
#include <ESP8266mDNS.h>
#include <FS.h>
#include <Wire.h>
#include <MD5Builder.h>
#include <Ticker.h>
#include <ArduinoJson.h>  // https://github.com/bblanchon/ArduinoJson

extern "C" {
#include "user_interface.h"
#include "espconn.h"
}
/*
   Для хранения конфигурации будем использовать SPIFFS
   config.h описывает простой способ чтения и записи
*/
#include "config.h"
config config;
/*
   датчик освещенности
   BH1750 https://github.com/claws/BH1750
*/
//#include <BH1750.h>
//BH1750 BH1750;
/*
   датчик влажности и температуры
   HDC1080 https://github.com/closedcube/ClosedCube_HDC1080_Arduino
   SI7021  https://github.com/LowPowerLab/SI7021
   HTU21D  https://github.com/sparkfun/SparkFun_HTU21D_Breakout_Arduino_Library
*/
//#include "ClosedCube_HDC1080.h"
//ClosedCube_HDC1080 HDC1080;

//#include <SI7021.h>
//SI7021 SI7021;

//#include "SparkFunHTU21D.h"
//HTU21D HTU21D;
/*
   датчик атмосферного давления и температуры
   BMP085 https://github.com/adafruit/Adafruit-BMP085-Library
*/
//#include <Adafruit_BMP085.h>
//Adafruit_BMP085 BMP085;
/*
   датчик атмосферного давления, температуры и влажности
   BME280 https://github.com/finitespace/BME280
*/
#include <BME280I2C.h>
BME280I2C BME;
/*
   OTA
*/
bool ota = false;
const char * otaHost = "espWeatherStation";
const char * otaPass = "";
/*
   Переменные для реализации автоматического выбора режима работы WiFi
*/
const bool client = false;
const bool ap = true;
bool modeWiFi;
unsigned long regimeChangeTimer = 0;
unsigned long lostConnectionTimer = 0;
bool eventSTAconnected = false;
byte failConnections = 0;
unsigned long connectionBreakTimer = 0;
/*
   DNS сервер/клиент
*/
const byte DNS_PORT = 53;
DNSServer dnsServer;
bool dnsClient = false;
const char* dnsClientName = "espws";
/*
   имя и ключ для cookies
*/
const String cookiesName = "esp-ws";
String cookiesKey;
/*
   счетчики и таймеры
*/
byte countAuthorizationFail = 0;          // счетчик неудачный попытов авторизации
unsigned long timerAuthorizationBan = 0;  // таймер для блокировки системы авторизации
unsigned long timerReadSensors = 0;       // таймер для переодического чтения показаний датчиков
unsigned long timerReset = 0;             // таймер для псевдо WD, сбрасывает контроллер при остановке loop()
unsigned long timerMQTT = 0;              // таймер для переодической отправки данных на MQTT сервер
unsigned long timerNarodmon = 0;          // таймер для переодической отправки данных на ресурс narodmon.ru
unsigned long timerDnsUpdate = 0;         // таймер для обновления записи на внешнем DNS сервере
unsigned long timerLogWrite = 0;          // таймер для переодической записи показаний в журнах

//===================================================начало==============================================================
/*
  Добавленные переменные для расчета разности давления каждые 2 часа
*/
unsigned long timerPressureDinamic = 0;   // таймер для двухчасового сравнения показания давления
double oldPress = 0;                      // предыдущее значение давления
double deltaPress = 0;                    // разница в показаниях давления через два часа
bool firstMessurePressureOld = true;      // переменная-флаг для записи первого измерения давления при старте модуля

//===================================================конец================================================================
/*
   i2c шина
   задейстованные пины SDA и SCL
*/
#define pin_sda 4 // default sda pin
#define pin_scl 5 // default scl pin
/*
   WEB сервер
*/
ESP8266WebServer webServer(80);
File spiffsUploadFile;
String spiffsUploadFileName = "";
/*
   MQTT
*/
WiFiClient espMqttClient;
PubSubClient mqtt(espMqttClient);
/*
   MD5 hash
*/
MD5Builder _md5;
String md5(String str) {
  _md5.begin();
  _md5.add(String(str));
  _md5.calculate();
  return _md5.toString();
}
/*
   структура и переменные для хранения показаний датчиков
*/
struct sensor {
  float data = 0;         // последние показания полученные от датчика
  bool  status = false;   // статус датчика
  float log[144];         // журнал показаний датчика
  int   logPosition = 0;  // следующая позиция в журнале для записи показаний датчика
};
sensor light;       // Уровень освещенности (lx)
sensor temperature; // Температура (*C)
sensor pressure;    // Давление (mm)
sensor humidity;    // Влажность (%)
/*
  прерывания
*/
Ticker interrupt1;
Ticker interrupt2;
/*
  структура для реализации индикации состояния устройства через мигание светодиода
*/
struct blink {
  byte current  = 0; // текущее состояние
  byte step     = 0; // битовый шаг
  byte mode[7]  = {  // интервалы индикации
    0B00000000, // Светодиод выключен
    0B00000001, // Короткая вспышка раз в секунду
    0B00000101, // Две короткие вспышки раз в секунду
    0B00010101, // Три короткие вспышки раз в секунду
    0B01010101, // Частые короткие вспышки (4 раза в секунду)
    0B11111111, // Горит постоянно
    0B00001111  // Мигание по 0.5 сек
  };
  byte pin_led = 2;       // пин к которому подключен светодиод (2 для ESP-12)
};
blink status;
/*
   Измерение напряжения
*/
ADC_MODE(ADC_VCC);
/*

*/
void setup() {
  /*
    Для теста
    Максимальное количество TCP соединений, по умолчанию 5
  */
  espconn_tcp_set_max_con(5);
  /*
     Инициализация пинов
  */
  pinMode(status.pin_led, OUTPUT);
  digitalWrite(status.pin_led, HIGH);
  /*
     Инициализация консоли
     Желательно полностью избавиться от вывода в консоль
  */
  Serial.begin(115200);
  Serial.println("\n");
  /*
     Возможность шить контроллер по сети через Arduino IDE
  */
  if (ota) {
    ArduinoOTA.setPort(8266);
    ArduinoOTA.setHostname(otaHost);
    ArduinoOTA.setPassword(otaPass);
    ArduinoOTA.begin();
    Serial.println("WARNING: OTA Enable!!!\n");
  }
  /*
     Информация о флеш памяти
  */
  flashInfo();
  /*
     Инициализация шины i2c
  */
  Wire.begin(pin_sda, pin_scl);
  /*
     Инициализация сенсоров на шине i2c
  */
  // BH1750
  #ifdef BH1750_h
    Wire.beginTransmission(0x23);
    if (Wire.endTransmission() == 0) {
      BH1750.begin();
      light.status = true;
    }
  #endif
  // HDC1080, SI7021 и HTU21D
  Wire.beginTransmission(0x40);
  if (Wire.endTransmission() == 0) {
    #ifdef _CLOSEDCUBE_HDC1080_h
      HDC1080.begin(0x40);
      temperature.status = humidity.status = true;
    #endif
    #ifdef si7021_h
      temperature.status = humidity.status = SI7021.begin(pin_sda, pin_scl);
    #endif
    #ifdef HTU21D_ADDRESS
      HTU21D.begin();
      temperature.status = humidity.status = true;
    #endif
  }
  // BMP085/180
  #ifdef ADAFRUIT_BMP085_H
    Wire.beginTransmission(0x77);
    if (Wire.endTransmission() == 0) {
      pressure.status = BMP085.begin();
    }
  #endif
  // BME280
  #ifdef TG_BME_280_I2C_H
    Wire.beginTransmission(0x76);
    if (Wire.endTransmission() == 0) {
      temperature.status = humidity.status = pressure.status = BME.begin();
    }
  #endif
  /*
     File system object (SPIFFS)
     https://github.com/esp8266/Arduino/blob/master/doc/filesystem.md#file-system-object-spiffs

     |--------------|-------|---------------|--|--|--|--|--|
     ^              ^       ^               ^     ^
     Sketch    OTA update   File system   EEPROM  WiFi config (SDK)
  */
  if (SPIFFS.begin()) {
    //SPIFFS.remove(config.file);
    Dir dir = SPIFFS.openDir("/");
    while (dir.next()) {
      String fileName = dir.fileName();
      size_t fileSize = dir.fileSize();
      Serial.printf("FS File: %s, size: %s\r\n", fileName.c_str(), formatBytes(fileSize).c_str());
    }
    Serial.println();
    if (!config.load()) config.save();
  } else ESP.restart();
  /*
     Поднимаем WiFi сеть
     В данной функции необходимо реализовывать логику переключения режимов ST и AP
  */
  WiFi.onEvent(WiFiEvent);
  if (!modeClient()) modeAP();
  /*
     webServer
     будем анализировать header заголовки запросов
  */
  // const char * headerkeys[] = {"User-Agent", "Cookie", "Accept-Encoding", "If-None-Match", "Origin"};
  const char * headerkeys[] = {"User-Agent", "Cookie", "Accept-Encoding", "If-None-Match"};
  size_t headerkeyssize = sizeof(headerkeys) / sizeof(char*);
  webServer.collectHeaders(headerkeys, headerkeyssize);
  /*
     webServer
     1. /api/sensors          (GET)  - обработка запросов показаний датчиков
     2. /api/sensors/log      (GET)  - список показаний датчиков за последнии 24 часа (144 points * 10 min = 24h)
     2. /api/settings         (POST) - работа с настройками
     3. /api/spiffs           (POST) - работа с файловой стистемой флеш памяти
     4. /api/spiffs           (GET)  - список файлов на флеш памяти
     5. /api/spiffs/delete    (POST) - удаление файлов web сервера
     6. /api/system/info      (POST) - системная информация
     7. /api/system/reboot    (POST) - перезагрузка микроконтроллера
     8. /api/system/hardReset (POST) - сброс всех настроек
     9. /api/system/i2c       (POST) - список устройств на i2c шине
    10. /api/system/update    (POST) - обновление прошивки микроконтроллера через http протокол
  */
  webServer.on("/api/sensors",          HTTP_GET,  web_api_sensors);
  webServer.on("/api/sensors/log",      HTTP_GET,  web_api_sensors_log);
  webServer.on("/api/settings",         HTTP_POST, web_api_settings);
  webServer.on("/api/spiffs",           HTTP_POST, web_api_spiffs_upload, web_api_spiffs_upload_header);
  webServer.on("/api/spiffs",           HTTP_GET,  web_api_spiffs_list);
  webServer.on("/api/spiffs/delete",    HTTP_POST, web_api_spiffs_delete);
  webServer.on("/api/system/info",      HTTP_POST, web_api_system_info);
  webServer.on("/api/system/reboot",    HTTP_POST, web_api_system_reboot);
  webServer.on("/api/system/hardReset", HTTP_POST, web_api_system_hardReset);
  webServer.on("/api/system/i2c",       HTTP_GET,  web_api_system_i2c_scaner);
  webServer.on("/api/system/update",    HTTP_POST, web_api_system_update, web_api_system_update_header);
  webServer.onNotFound(web_error_404);

  // генерируем ключ пустышку
  cookiesKey = salt(32);
  // устанавливаем прерывания
  interrupt1.attach(10, hangTest);
  interrupt2.attach_ms(125, displayStatus);
  // первое чтение данных с датчиков
  readSensors(); sensorLogWrite();
}
/*
  Индикация состояния устройства
*/
void displayStatus() {
  if (status.step > 7) status.step = 0;
  if (status.current) digitalWrite(status.pin_led, (status.mode[status.current] >> status.step++ & 1) ? LOW : HIGH);
}
/*
  Сброс контроллера при offline более 120 секунд
*/
void hangTest() {
  if (millis() - timerReset > 1200000) {
    status.current = 5;
    ESP.restart();
  }
}
void hangTestReset() {
  timerReset = millis();
}
/*
  Обработчик событий WiFi
*/
void WiFiEvent(WiFiEvent_t event) {
  switch (event) {
    /*
      Используемые события
    */
    case WIFI_EVENT_STAMODE_CONNECTED:
      status.current = 1;
      eventSTAconnected = true;
      if (lostConnectionTimer != 0) lostConnectionTimer = 0;
      if (regimeChangeTimer != 0) regimeChangeTimer = 0;
      Serial.println("connected successfully");
      break;
    case WIFI_EVENT_STAMODE_DISCONNECTED:
      status.current = 0;
      if (lostConnectionTimer == 0 and eventSTAconnected) {
        lostConnectionTimer = millis();
        eventSTAconnected = false;
        Serial.println("connection lost");
      }
      break;
    case WIFI_EVENT_SOFTAPMODE_STACONNECTED:
      Serial.printf("a new connection, all connected users: %i\n", WiFi.softAPgetStationNum());
      break;
    case WIFI_EVENT_SOFTAPMODE_STADISCONNECTED:
      Serial.printf("disconnection, all connected users: %i\n", WiFi.softAPgetStationNum());
      break;
    /*
       Не используемые события
    */
    case WIFI_EVENT_STAMODE_AUTHMODE_CHANGE:
    case WIFI_EVENT_STAMODE_DHCP_TIMEOUT:
    case WIFI_EVENT_STAMODE_GOT_IP:
    case WIFI_EVENT_SOFTAPMODE_PROBEREQRECVED:
    case WIFI_EVENT_MODE_CHANGE:
      break;
  }
}
/*
   WiFi
   Режим клиента
*/
bool modeClient() {
  Serial.println("WiFi mode client");
  if (config.client_ssid.length()) {
    WiFi.mode(WIFI_STA);
    WiFi.begin(config.client_ssid.c_str(), config.client_pass.length() ? config.client_pass.c_str() : NULL);
  } else return false;
  Serial.println("connecting to " + config.client_ssid);
  unsigned long time = millis();
  while (!WiFi.isConnected()) {
    delay(10);
    if (millis() - time > 30000) {
      Serial.println("connecting fail");
      WiFi.disconnect();
      delay(10);
      return false;
    }
  }
  if (!WiFi.getAutoConnect()) {
    WiFi.setAutoConnect(true);
    WiFi.setAutoReconnect(true);
  }

  Serial.println();
  Serial.print("mac: ");
  Serial.println(WiFi.macAddress());
  Serial.print("hostname: ");
  Serial.println(WiFi.hostname());
  Serial.print("ip: ");
  Serial.println(WiFi.localIP());
  Serial.print("subnet mask: ");
  Serial.println(WiFi.subnetMask());
  Serial.print("gateway: ");
  Serial.println(WiFi.gatewayIP());
  Serial.print("dns server: ");
  Serial.println(WiFi.dnsIP());
  Serial.println();

  webServer.begin();

  if ((dnsClient = MDNS.begin(dnsClientName)) == true) {
    MDNS.addService("http", "tcp", 80);
    timerDnsUpdate = millis();
  }

  modeWiFi = client;
  status.current = 1;
  return true;
}
/*
   WiFi
   Режим точки доступа
*/
bool modeAP() {
  Serial.println("WiFi mode AP\n");
  if (config.softap_ssid.length()) {
    IPAddress apIP(10, 10, 10, 1);
    IPAddress netMsk(255, 255, 255, 0);

    WiFi.mode(WIFI_AP);
    WiFi.softAPConfig(apIP, apIP, netMsk);
    WiFi.softAP(config.softap_ssid.c_str(), config.softap_pass.length() ? config.softap_pass.c_str() : NULL);

    dnsServer.setErrorReplyCode(DNSReplyCode::NoError);
    dnsServer.start(DNS_PORT, String(dnsClientName) + ".local", apIP);
  } else return false;

  webServer.begin();
  modeWiFi = ap;
  regimeChangeTimer = millis();
  status.current = 2;

  if (dnsClient) {
    dnsClient = false;
    timerDnsUpdate = 0;
  }

  return true;
}
/*
   HTTP
   Обновление прошивки микроконтроллера через web интерфейс
   HTTPMethod
*/
void web_api_system_update() {
  webServer.sendHeader("Connection", "close");
  if (authorized()) {
    if (!Update.hasError()) {
      webServer.sendHeader("Set-Cookie", cookiesName + "=0; Path=/; Expires=Thu, 02 Apr 1987 00:00:01 GMT");
      webServer.send(200, "text/plain", "{\"status\":true}");
      ESP.restart();
    } else webServer.send(200, "text/plain", "{\"status\":false,\"error\":" + String(Update.getError()) + "}");
  }
  else webServer.send(200, "text/plain", "{\"status\":false}");
}
/*
   HTTP
   Обновление прошивки микроконтроллера через web интерфейс
   THandlerFunction
*/
void web_api_system_update_header() {
  if (!authorized()) return;
  HTTPUpload& upload = webServer.upload();
  if (upload.status == UPLOAD_FILE_START) {
    //Serial.setDebugOutput(true);
    WiFiUDP::stopAll();
    Serial.printf("Firmware update. MD5: %s\n", upload.filename.c_str()); // MD5 in the update file name
    uint32_t maxSketchSpace = (ESP.getFreeSketchSpace() - 0x1000) & 0xFFFFF000;
    if (!Update.begin(maxSketchSpace)) Update.printError(Serial);
  } else if (upload.status == UPLOAD_FILE_WRITE) {
    if (Update.write(upload.buf, upload.currentSize) != upload.currentSize) Update.printError(Serial);
  } else if (upload.status == UPLOAD_FILE_END) {
    Update.setMD5(upload.filename.c_str());
    if (Update.end(true)) Serial.printf("Update Success: %u\nRebooting...\n", upload.totalSize);
    else Update.printError(Serial);
    //Serial.setDebugOutput(false);
  } else if (upload.status == UPLOAD_FILE_ABORTED) Update.end();
}
/*
   Основной цикл программы
*/
void loop() {
  if (ota) ArduinoOTA.handle();
  // Сбрасываем псевдо сторожевой таймер
  hangTestReset();
  // Обрабатываем пользовательские http запросы
  webServer.handleClient();
  // Запросы к DNS серверу ESP8266 обрабатываем только в режиме AP
  if (modeWiFi == ap) dnsServer.processNextRequest();
  // Запросы к внешнему DNS серверу отправляем раз в час и только в режиме STA
  if (modeWiFi == client and dnsClient and (millis() > timerDnsUpdate + 3600000 or timerDnsUpdate > millis())) {
    MDNS.update();
    timerDnsUpdate = millis();
  }
  // Переводим контроллер для работы в режиме AP при потери связи с домашним WiFi
  if (modeWiFi == client and lostConnectionTimer != 0 and (millis() > lostConnectionTimer + 30000 or lostConnectionTimer > millis())) {
    lostConnectionTimer = 0;
    webServer.stop();
    WiFi.disconnect();
    modeAP();
  }
  // При потери связи с базовой станцией переходим в режим точки доступа и переодически ищем в эфире домашнюю сеть
  if (modeWiFi != client and regimeChangeTimer != 0 and config.client_ssid.length() and connectionBreakTimer == 0) {
    if (millis() > regimeChangeTimer + 30000 or regimeChangeTimer > millis()) {
      Serial.println("scanning of radio in search of a \"" + config.client_ssid + "\" access point");
      int n = WiFi.scanNetworks(false, true);
      if (n > 0) {
        bool find = false;
        String protect, hidden;
        for (int i = 0; i < n; ++i) {
          if (WiFi.SSID(i) == String(config.client_ssid)) find = true;
          protect = WiFi.encryptionType(i) == ENC_TYPE_NONE ? "<!>" : "protected";
          hidden = WiFi.isHidden(i) ? "*hidden" : " ";
          Serial.printf("%i: %s (%idBm) %s %s\n", i + 1, WiFi.SSID(i).c_str(), WiFi.RSSI(i), protect.c_str(), hidden.c_str());
        }
        if (find) {
          status.current = 0;
          Serial.println("wireless network is found, connect ...");
          dnsServer.stop();
          webServer.stop();
          WiFi.softAPdisconnect();
          if (!modeClient()) {
            modeAP();
            if (++failConnections >= 1) {
              connectionBreakTimer = millis();
              Serial.println("The next connection attempt will be made after 10 minutes!");
            }
          } else failConnections = 0;
        }
      } else Serial.println("no networks are available");
      Serial.println();
      regimeChangeTimer = millis();
    }
  }
  // Разрешаем повторные подключения к базовой станции после 5 минут ожидания после 3 неудачных подключений подряд
  if (connectionBreakTimer != 0 and (millis() > connectionBreakTimer + 5 * 60000 or connectionBreakTimer > millis())) connectionBreakTimer = failConnections = 0;
  // Снимаем ограничение на доступ к системе авторизации
  if (timerAuthorizationBan != 0 and (millis() > timerAuthorizationBan + 60000 or timerAuthorizationBan > millis())) countAuthorizationFail = timerAuthorizationBan = 0;
  // Читаем показания с датчиков каждые 5 сек
  if (millis() > timerReadSensors + 5000 or timerReadSensors > millis()) readSensors();

  //=======================================================начало=======================================================================
  
  // Запоминаем первое измерение давления для даальнейшего сравнения каждые 2 часа
  if (firstMessurePressureOld){
    oldPress = pressure.data;
    firstMessurePressureOld = false;
     }
  // Если прошло 2 часа, вычисляем дельту давления
  if (millis() - timerPressureDinamic > 7200000 or timerPressureDinamic > millis()) pressureDinamic();

  //========================================================конец=======================================================================
  
  // Записываем показания с датчикв в журнал каждые 10 минут (144 points * 10 min = 24h)
  if (millis() > timerLogWrite + 600000 or timerLogWrite > millis()) sensorLogWrite();
  // Отправка данных на внешние ресурсы только в режиме клиента
  if (modeWiFi == client) {
    // Отправка данных на ресурс narodmon.ru с интервалом 10 минут
    if (config.narodmon_id.length()) {
      if (millis() - timerNarodmon > 600000 or timerNarodmon > millis()) {
        HTTPClient httpClient;
        String query;
        query += "&L1=" + String(light.data);
        query += "&T1=" + String(temperature.data);
        query += "&H1=" + String(humidity.data);
        query += "&P1=" + String(pressure.data);
        httpClient.begin("narodmon.ru", 80, "/get?ID=" + config.narodmon_id + query + "&NAME=ESP8266 (iT4iT.CLUB)");
        httpClient.setUserAgent("weather station (ESP8266) iT4iT.CLUB");
        httpClient.setTimeout(5000);
        httpClient.GET();
        httpClient.end();
        timerNarodmon = millis();
      }
    }
    // Отправка данных на MQTT сервер с интервалом 5 минут
    if (config.mqtt_server.length()) {
      if (millis() - timerMQTT > 60000 or timerMQTT > millis()) {
        mqttSendData();
        timerMQTT = millis();
      }
    }
  }
}
/*
    Функция отвечает за отправку данных на MQTT сервер
*/
bool mqttSendData() {
  mqtt.setServer(config.mqtt_server.c_str(), 1883);
  if (config.mqtt_login.length()) mqtt.connect("weather station", config.mqtt_login.c_str(), config.mqtt_pass.c_str());
  else mqtt.connect("weather station");
  if (mqtt.connected()) {
    
//==========================================================начало=======================================================================   
/*
 * Отправка URL в зависимости от изменения давления
*/
      
      //если дельта в этом диапазоне, отправляем URL картинки "null" (атмосферное давление без значительных возмущений)
      if((deltaPress >= 0)&(deltaPress < 0.75)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500ab/fz2lSD1bs1I.jpg");
      
      
      //если дельта в этом диапазоне, отправляем URL картинки "up1"  (давление растет со скоростью 1 гПа/2ч.)
      if((deltaPress >= 0.75)&(deltaPress < 1.5)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500b2/MbRtTUknywc.jpg"); 
      
      //если дельта в этом диапазоне, отправляем URL картинки "up2"  (давление растет со скоростью 2 гПа/2ч.)
      if((deltaPress >= 1.5)&(deltaPress < 2.25)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500b9/_VPttV2p9Uw.jpg"); 
      
      //если дельта в этом диапазоне, отправляем URL картинки "up3"  (давление растет со скоростью 3 гПа/2ч.)
      if((deltaPress >= 2.25)&(deltaPress < 3)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500c0/AAbmzwbFTjk.jpg"); 

      //если дельта в этом диапазоне, отправляем URL картинки "up4"  (давление растет со скоростью 4 и более гПа/2ч.)
      if(deltaPress >= 3) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500c7/-UX1fvT_F5Y.jpg"); 

      //если дельта в этом диапазоне, отправляем URL картинки "null" (атмосферное давление без значительных возмущений)
      if((deltaPress <= 0)&(deltaPress > -0.75))  mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500ab/fz2lSD1bs1I.jpg"); 
      
      //если дельта в этом диапазоне, отправляем URL картинки "down1"  (давление падает со скоростью 1 гПа/2ч.)
      if((deltaPress <= -0.75)&(deltaPress > -1.5)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/5008f/oSdt4OteOAI.jpg"); 
      
      //если дельта в этом диапазоне, отправляем URL картинки "down2"  (давление падает со скоростью 2 гПа/2ч.)
      if((deltaPress <= -1.5)&(deltaPress > -2.25)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/50096/2UgB7TLJRhQ.jpg"); 
      
      //если дельта в этом диапазоне, отправляем URL картинки "down3"  (давление падает со скоростью 3 гПа/2ч.)
      if((deltaPress <= -2.25)&(deltaPress > -3)) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/5009d/atgIrwg2fDs.jpg");
       
      //если дельта в этом диапазоне, отправляем URL картинки "down4"  (давление падает со скоростью 4 и более гПа/2ч.)
      if(deltaPress <= -3) mqttPublishURL("deltapressure", "https://pp.userapi.com/c840321/v840321283/500a4/HKWuw2h52X0.jpg");
    
    
//=============================================================конец===============================================================================    
    
    
    mqttPublish("light", light.data);
    mqttPublish("temperature", temperature.data);
    mqttPublish("humidity", humidity.data);
    mqttPublish("pressure", pressure.data);
   
    //===================================================начало=================================================================
    
    mqttPublish("deltapressFloat", deltaPress);        //для отладки отправим дельту в MQTT
    
   //====================================================конец=======================================================================
    
    mqtt.disconnect();
    return true;
  }
  return false;
}
bool mqttPublish(String topic, float data) {
  if (config.mqtt_path.length()) topic = config.mqtt_path + "/" + topic;
  return mqtt.publish(topic.c_str(), String(data).c_str(), true);
}

//=======================================================начало=======================================================

/*
Негуманный код для отправки URL по MQTT
*/
bool mqttPublishURL(String topic, String data) {
  if (config.mqtt_path.length()) topic = config.mqtt_path + "/" + topic;
  return mqtt.publish(topic.c_str(), String(data).c_str(), true);
}

//=======================================================конец========================================================

/*
   Функция отвечает за авторизацию пользователя с использованием cookie
   Set-Cookie: NAME=VALUE; expires=DATE; path=PATH; domain=DOMAIN_NAME; secure
   https://ru.wikipedia.org/wiki/HTTP_cookie
*/
bool authorized() {
  // Если есть подозрения на взлом, отбиваем все попытки авторизации
  if (timerAuthorizationBan != 0 and countAuthorizationFail > 5) return false;
  // Переменные для идентификации пользователя
  String clientIP = webServer.client().remoteIP().toString();
  String userAgent;
  /*
     String origin = webServer.header("Origin");
     webServer.sendHeader("Access-Control-Allow-Origin", origin);
     webServer.sendHeader("Access-Control-Allow-Credentials", "true");
  */
  // ищем id браузера клиента
  if (webServer.hasHeader("User-Agent")) userAgent = webServer.header("User-Agent");
  else return false;
  // ищем cookies
  if (webServer.hasHeader("Cookie")) {
    String cookie = webServer.header("Cookie");
    if (cookie.length() > cookiesName.length() + 33) return false;
    if (cookie.indexOf(cookiesName + "=" + cookiesKey) != -1) return true;
    else {
      // Неоднократное срабатывание говорит о попытке взлома путем подбора ключа cookies
      countAuthorizationFail += 1;
      timerAuthorizationBan = millis();
      // Пытаемся удалить cookie у пользователя
      webServer.sendHeader("Set-Cookie", cookiesName + "=0; Path=/; Expires=Thu, 02 Apr 1987 00:00:01 GMT");
      return false;
    }
  }
  // обрабатываем форму авторизации
  if (webServer.hasArg("login") and webServer.hasArg("password")) {
    if (webServer.arg("login") == config.admin_login and webServer.arg("password") == config.admin_pass) {
      cookiesKey = md5(clientIP + salt(32) + userAgent);
      webServer.sendHeader("Set-Cookie", cookiesName + "=" + cookiesKey + "; Path=/");
      return true;
    }
    else {
      // Неоднократное срабатывание говорит о брутфорсе
      countAuthorizationFail += 1;
      timerAuthorizationBan = millis();
    }
  }
  // шьем бороду
  return false;
}
/*
   Функция считывающая показания сенсоров через установленный интервал времени
*/
void readSensors() {
  timerReadSensors = millis();
  /* BH1750 */
  #ifdef BH1750_h
    if (light.status) light.data = BH1750.readLightLevel();
  #endif
  /* HDC1080 */
  #ifdef _CLOSEDCUBE_HDC1080_h
    if (temperature.status) temperature.data = HDC1080.readTemperature();
    if (humidity.status) humidity.data = HDC1080.readHumidity();
  #endif
  /* SI7021 */
  #ifdef si7021_h
    if (temperature.status) temperature.data = SI7021.getCelsiusHundredths() * 0.01;
    if (humidity.status) humidity.data = SI7021.getHumidityPercent();
  #endif
  /* HTU21D */
  #ifdef HTU21D_ADDRESS
    if (temperature.status) temperature.data = HTU21D.readTemperature();
    if (humidity.status) humidity.data = HTU21D.readHumidity();
  #endif
  /* BMP085 */
  #ifdef ADAFRUIT_BMP085_H
    if (pressure.status) pressure.data = BMP085.readPressure() / 133.3;
  #endif
  /* BME280 */
  #ifdef TG_BME_280_I2C_H
    if (temperature.status) BME.read(pressure.data, temperature.data, humidity.data, BME280::TempUnit_Celsius, BME280::PresUnit_torr);
  #endif
}
/*
   HTTP API
   Ответ на запрос показаний датчиков
*/
void web_api_sensors() {
  //Serial.print(webServer.client().remoteIP().toString() + ": ");
  //Serial.print("handleFileRead: /api/sensors");
  String answer;

  StaticJsonBuffer<200> jsonBuffer;
  JsonObject& sensor = jsonBuffer.createObject();

  sensor["light"] = light.data;
  sensor["temperature"] = temperature.data;
  sensor["humidity"] = humidity.data;
  sensor["pressure"] = pressure.data;

  sensor.printTo(answer);
  webServer.sendHeader("Cache-Control", "no-store, no-cache");
  webServer.send(200, "text/plain", answer);
}
/*
  Функция добавляет новую запись в журнал и помечает следующую позицию для перезаписи
*/
void sensorLogUpdate(struct sensor * sensor, String name = "") {
  unsigned int logSize = (sizeof(sensor->log) / sizeof(float *)) - 1;
  if (name.length()) Serial.print("LU " + String(sensor->logPosition) + "x" + String(logSize));
  sensor->log[sensor->logPosition++] = sensor->data;
  if (sensor->logPosition > logSize) sensor->logPosition = 0;
  if (name.length()) {
    Serial.print("\n" + name + ": ");
    for (int i = 0; i <= logSize; i++) Serial.print(" [" + String(i) + "]" + sensorDelFloatNull(sensor->log[i]));
    Serial.println();
  }
}
/*
  Функция инициализирует добавление новых записей в журналы датчиков
*/
void sensorLogWrite() {
  timerLogWrite = millis();

  sensorLogUpdate(&light/*, "light"*/);
  sensorLogUpdate(&temperature/*, "temperature"*/);
  sensorLogUpdate(&humidity/*, "humidity"*/);
  sensorLogUpdate(&pressure/*, "pressure"*/);
}
/*
  Функция формирует правильную последовательность показаний с учётом текущего смещения 0 отметки в журнале
*/
String sensorLogRead(struct sensor * sensor) {
  String log;
  unsigned int logSize = (sizeof(sensor->log) / sizeof(float *)) - 1;
  for (int i = sensor->logPosition; i <= logSize; i++) log += (log.length() ? "," : "") + sensorDelFloatNull(sensor->log[i]);
  for (int i = 0; i < sensor->logPosition; i++) log += (log.length() ? "," : "") + sensorDelFloatNull(sensor->log[i]);
  return log;
}
/*
  Функция необходима для удаления лишних символов при передаче показаний журнала в web интерфейс
*/
String sensorDelFloatNull(float data) {
  if (data == 0) return "0";
  if (data - (int)data == 0) return String((int)data);
  return String(data);
}
/*
  Функция расчета абсолютной влажности воздуха (грамм воды на один кубический метр воздуха)
  https://carnotcycle.wordpress.com/2012/08/04/how-to-convert-relative-humidity-to-absolute-humidity/
*/
float sensorsAbsoluteHumidity(float t, float h) {
  double tmp = pow(2.718281828, (17.67 * t) / (t + 243.5));
  return (6.112 * tmp * h * 2.1674) / (273.15 + t);
}
/*
  HTTP_API
  Журнал показаний за последние сутки
*/
void web_api_sensors_log() {
  String json;
  unsigned long timeAdjustment = (millis() < timerLogWrite) ? 0 : millis() - timerLogWrite;

  json += "\"timeAdjustment\":" + String(timeAdjustment) + ",";
  json += "\"light\":[" + sensorLogRead(&light) + "],";
  json += "\"temperature\":[" + sensorLogRead(&temperature) + "],";
  json += "\"humidity\":[" + sensorLogRead(&humidity) + "],";
  json += "\"pressure\":[" + sensorLogRead(&pressure) + "]";

  webServer.sendHeader("Cache-Control", "no-store, no-cache");
  webServer.send(200, "text/plain", "{" + json + "}");
}
/*
   HTTP API
   Доступ к настройкам только для автооризованных пользователей
*/
void web_api_settings() {
  webServer.sendHeader("Cache-Control", "no-store, no-cache");
  if (!authorized()) {
    webServer.send(200, "text/html", "{\"status\":false}");
    return;
  }
  if (webServer.hasArg("save")) {
    if (!config.save(webServer.arg("save"))) {
      webServer.send(200, "text/html", "{\"status\":false}");
      return;
    }
  }
  webServer.send(200, "text/html", config.jsonSecurityLoad());
}
/*
   HTTP API
   Загрузка файлов Web сервера по http протоколу
   HTTPMethod
*/
void web_api_spiffs_upload() {
  if (!authorized()) webServer.send(404, "text/plain", "404 Not Found");
}
/*
   HTTP API
   Загрузка файлов Web сервера по http протоколу
   THandlerFunction
*/
void web_api_spiffs_upload_header() {
  hangTestReset();
  HTTPUpload& upload = webServer.upload();
  if (upload.status == UPLOAD_FILE_START) {
    if (!authorized()) {
      webServer.send(404, "text/plain", "404 Not Found");
      return;
    }
    WiFiUDP::stopAll();
    String filename = upload.filename;
    if (!filename.startsWith("/")) filename = "/" + filename;
    if (filename == config.file) {  // <- НЕ УДАЛЯТЬ! DO NOT DELETE! 请勿卸下！ENTFERNEN SIE NICHT! NE PAS RETIRER!
      webServer.send(404, "text/plain", "404 Not Found");
      return;
    }
    Serial.print("handleFileUpload Name: "); Serial.println(filename);
    spiffsUploadFile = SPIFFS.open(filename, "w");
    filename = String();
  } else if (upload.status == UPLOAD_FILE_WRITE) {
    if (spiffsUploadFile) spiffsUploadFile.write(upload.buf, upload.currentSize);
  } else if (upload.status == UPLOAD_FILE_END) {
    if (spiffsUploadFile) {
      spiffsUploadFile.close();
      Serial.print("handleFileUpload Size: "); Serial.println(formatBytes(upload.totalSize));
      webServer.send(200, "text/plain", "{\"status\":true}");
    }
  } else if (upload.status == UPLOAD_FILE_ABORTED) {
    if (spiffsUploadFile) {
      spiffsUploadFile.close();
      Serial.print("handleFileUpload: aborted");
      webServer.send(200, "text/plain", "{\"status\":false}");
    }
  }
  /*
    String tmpFileName = "/tmp";
    hangTestReset();

    HTTPUpload& upload = webServer.upload();
    if (upload.status == UPLOAD_FILE_START) {
    if (!authorized()) {
      webServer.send(404, "text/plain", "404 Not Found");
      return;
    }
    WiFiUDP::stopAll();
    spiffsUploadFileName = upload.filename;
    if (!spiffsUploadFileName.startsWith("/")) spiffsUploadFileName = "/" + spiffsUploadFileName;
    Serial.print("handleFileUpload Name: "); Serial.println(spiffsUploadFileName);
    spiffsUploadFile = SPIFFS.open(tmpFileName, "w");

    } else if (upload.status == UPLOAD_FILE_WRITE) {
    if (spiffsUploadFile) spiffsUploadFile.write(upload.buf, upload.currentSize);

    } else if (upload.status == UPLOAD_FILE_END) {
    if (spiffsUploadFile) spiffsUploadFile.close();
    if (SPIFFS.exists(spiffsUploadFileName)) SPIFFS.remove(spiffsUploadFileName);
    SPIFFS.rename(tmpFileName, spiffsUploadFileName);
    SPIFFS.remove(tmpFileName);
    Serial.print("handleFileUpload Size: "); Serial.println(formatBytes(upload.totalSize));
    webServer.send(200, "text/plain", "{\"status\":true}");
    spiffsUploadFileName = String("");

    } else if (upload.status == UPLOAD_FILE_ABORTED) {
    if (spiffsUploadFile) spiffsUploadFile.close();
    if (SPIFFS.exists(tmpFileName)) SPIFFS.remove(tmpFileName);
    Serial.print("handleFileUpload: aborted");
    webServer.send(200, "text/plain", "{\"status\":false}");
    spiffsUploadFileName = String("");
    }
  */
}
/*
   HTTP API
   Получение списка файлов на Flash памяти
*/
void web_api_spiffs_list() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }

  String output = "";
  int count = 0;
  uint32_t realSize = ESP.getFlashChipRealSize();

  Dir dir = SPIFFS.openDir("/");
  while (dir.next()) {
    String fileName = dir.fileName().substring(1);
    size_t fileSize = dir.fileSize();
    if ("/" + fileName != config.file) {
      if (++count > 1) output += ",";
      output += "{\"name\":\"" + fileName + "\",\"size\":" + String(fileSize) + "}";
    }
  }
  webServer.send(200, "text/plain", "{\"count\":" + String(count) + ",\"flashSize\":" + String(realSize) + ",\"list\":[" + output + "]}");
}
/*
   HTTP API
   Удаление файла с Flash памяти
*/
void web_api_spiffs_delete() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }
  if (!webServer.hasArg("path")) {
    webServer.send(500, "text/plain", "Bad arguments");
    return;
  }

  String path = webServer.arg("path");
  if (!path.startsWith("/")) path = "/" + path;
  if (path.endsWith("/")) path += "index.htm";
  if (path == config.file) {  // <- НЕ УДАЛЯТЬ! DO NOT DELETE! 请勿卸下！ENTFERNEN SIE NICHT! NE PAS RETIRER!
    webServer.send(500, "text/plain", "Bad arguments");
    return;
  }
  if (SPIFFS.exists(path)) {
    SPIFFS.remove(path);
    webServer.send(200, "text/plain", "{\"status\":true}");
  } else webServer.send(200, "text/plain", "{\"status\":false}");
  Serial.println("handleFileDelete: " + path);
}
/*
   HTTP API
   Получение основной информации о микроконтроллере и его программе
*/
void web_api_system_info() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }

  String answer;
  answer += "\"mac\":\"" + String(WiFi.macAddress()) + "\",";                // uint8_t (array[6])
  answer += "\"vcc\":\"" + String(ESP.getVcc() * 0.001) + " V\",";           // uint16_t
  answer += "\"freeHeap\":" + String(ESP.getFreeHeap()) + ",";               // uint32_t
  answer += "\"chipId\":" + String(ESP.getChipId()) + ",";                   // uint32_t
  answer += "\"sdkVersion\":\"" + String(ESP.getSdkVersion()) + "\",";       // const char *
  answer += "\"coreVersion\":\"" + ESP.getCoreVersion() + "\",";             //
  answer += "\"cpuFreqMHz\":\"" + String(ESP.getCpuFreqMHz()) + " MHz\",";   // uint8_t

  answer += "\"flashRealSize\":" + String(ESP.getFlashChipRealSize()) + ","; // uint32_t
  answer += "\"flashChipId\":" + String(ESP.getFlashChipId()) + ",";         // uint32_t
  answer += "\"flashChipSpeed\":\"" + String(int(ESP.getFlashChipSpeed() * 0.000001)) + " MHz\","; // uint32_t

  answer += "\"sketchSize\":" + String(ESP.getSketchSize()) + ",";           // uint32_t
  answer += "\"sketchMD5\":\"" + ESP.getSketchMD5() + "\",";                 //
  answer += "\"freeSketchSpace\":" + String(ESP.getFreeSketchSpace()) + ","; // uint32_t

  answer += "\"resetReason\":\"" + ESP.getResetReason() + "\",";             //
  answer += "\"resetInfo\":\"" + ESP.getResetInfo() + "\",";                 //

  answer += "\"bootVersion\":\"" + String(ESP.getBootVersion()) + "\"";      // uint8_t
  //answer += "\"bootMode\":\"" + String(ESP.getBootMode()) + "\"";          // uint8_t

  webServer.send(200, "text/plain", "{" + answer + "}");
}
/*
   HTTP API
   Перезагрузка
*/
void web_api_system_reboot() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }
  status.current = 5;
  webServer.send(200, "text/plain", "{\"status\":true}");
  ESP.restart();
}
/*
  HTTP API
  Сброс настроек устройства
*/
void web_api_system_hardReset() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }
  status.current = 5;
  webServer.send(200, "text/plain", "{\"status\":true}");
  config.remove();
  WiFi.mode(WIFI_OFF);
  delay(10000);
  ESP.restart();
}
/*
   HTTP API
   Сканер i2c шины
   http://playground.arduino.cc/Main/I2cScanner
*/
void web_api_system_i2c_scaner() {
  if (!authorized()) {
    webServer.send(404, "text/plain", "404 Not Found");
    return;
  }
  byte error, address, count = 0;
  String answer;
  for (address = 1; address < 127; address++) {
    Wire.beginTransmission(address);
    error = Wire.endTransmission();
    if (error == 0 or error == 4) {
      answer += answer.length() ? ",\"" : "{\"";
      if (address < 16) answer += '0';
      answer += String(address, HEX) + "\":" + (!error ? '1' : '0');
      count++;
    }
  }
  if (answer.length()) answer += "}";
  webServer.send(200, "text/plain", "{\"count\":" + String(count) + ", \"list\":[" + answer + "]}");
}
/*
   HTTP
   error 404
*/
void web_error_404() {
  if (!fs_handle_file(webServer.uri())) webServer.send(404, "text/plain", "404 Not Found");
}
/*
   File system object (SPIFFS)
   Обработчик запросов на доступ к реально существующим файлам
*/
bool fs_handle_file(String path) {
  webServer.sendHeader("Server", "ESP8266");
  webServer.sendHeader("Cache-Control", "no-transform, private, max-age=86400, s-maxage=86400");
  // [!] Доступ к конфигу только через API [!]
  if (path == config.file) return false; // <- НЕ УДАЛЯТЬ! DO NOT DELETE! 请勿卸下！ENTFERNEN SIE NICHT! NE PAS RETIRER!
  if (path.endsWith("/")) path += "index.htm";
  Serial.print(webServer.client().remoteIP().toString() + ": " + path);
  String fileType = fs_getContentType(path);
  if (webServer.hasHeader("Accept-Encoding")) {
    String encoding = webServer.header("Accept-Encoding");
    if (encoding.indexOf("gzip") != -1 and SPIFFS.exists(path + ".gz")) path += ".gz";
  }
  if (SPIFFS.exists(path)) {
    File file = SPIFFS.open(path, "r");
    size_t fSize = file.size();
    // [!] Подобие системы кеширвоания [!]
    // [!] Необходимо быть уверенным, что размер файла после модификации изменился [!]
    webServer.sendHeader("ETag", String(fSize));
    if (webServer.hasHeader("If-None-Match")) {
      String Etag = webServer.header("If-None-Match");
      if (Etag == String(fSize)) {
        webServer.send(304, "text/plain", "Not Modified");
        file.close();
        Serial.println(" 304, Etag " + Etag);
        return true;
      }
    }
    Serial.println(" 200");
    webServer.streamFile(file, fileType);
    file.close();
    return true;
  }
  Serial.println(" 404");
  return false;
}
/*
   File system object (SPIFFS)
   Функция пересчета размера файлов
*/
String formatBytes(size_t bytes) {
  if (bytes < 1024) return String(bytes) + "B";
  else if (bytes < pow(1024, 2)) return String(bytes / 1024) + "KB";
  else if (bytes < pow(1024, 3)) return String(bytes / pow(1024, 2)) + "MB";
  else return String(bytes / pow(1024, 3)) + "GB";
}
/*
   File system object (SPIFFS)
   Функция участвует в генерации заголовка при ответе клиенту
*/
String fs_getContentType(String& path) {
  if (path.endsWith(".html"))       return "text/html";
  else if (path.endsWith(".htm"))   return "text/html";
  else if (path.endsWith(".css"))   return "text/css";
  else if (path.endsWith(".txt"))   return "text/plain";
  else if (path.endsWith(".js"))    return "application/javascript";
  else if (path.endsWith(".png"))   return "image/png";
  else if (path.endsWith(".gif"))   return "image/gif";
  else if (path.endsWith(".jpg"))   return "image/jpeg";
  else if (path.endsWith(".ico"))   return "image/x-icon";
  else if (path.endsWith(".svg"))   return "image/svg+xml";
  else if (path.endsWith(".ttf"))   return "application/x-font-ttf";
  else if (path.endsWith(".otf"))   return "application/x-font-opentype";
  else if (path.endsWith(".woff"))  return "application/font-woff";
  else if (path.endsWith(".woff2")) return "application/font-woff2";
  else if (path.endsWith(".eot"))   return "application/vnd.ms-fontobject";
  else if (path.endsWith(".sfnt"))  return "application/font-sfnt";
  else if (path.endsWith(".xml"))   return "text/xml";
  else if (path.endsWith(".pdf"))   return "application/pdf";
  else if (path.endsWith(".zip"))   return "application/zip";
  else if (path.endsWith(".gz"))    return "application/x-gzip";
  return "application/octet-stream";
  //return "text/plain";
}
/*
   Информация о флеш памяти
*/
void flashInfo() {
  uint32_t realSize = ESP.getFlashChipRealSize();
  uint32_t ideSize = ESP.getFlashChipSize();
  FlashMode_t ideMode = ESP.getFlashChipMode();

  Serial.printf("Flash real id:   %08X\r\n", ESP.getFlashChipId());
  Serial.printf("Flash real size: %u\r\n", realSize);
  Serial.printf("Flash ide size: %u\r\n", ideSize);
  Serial.printf("Flash ide speed: %u\r\n", ESP.getFlashChipSpeed());
  Serial.printf("Flash ide mode:  %s\r\n", (ideMode == FM_QIO ? "QIO" : ideMode == FM_QOUT ? "QOUT" : ideMode == FM_DIO ? "DIO" : ideMode == FM_DOUT ? "DOUT" : "UNKNOWN"));
  Serial.printf("Flash Chip configuration %s\r\n", ideSize != realSize ? "wrong!" : "ok");
  Serial.println();
}
/*
   Генератор соли
*/
String salt(int strlen) {
  String answer;
  char simbol[82] = {
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z',
    'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z',
    '!', '@', '#', '$', '%', '^', '&', '*', '(', ')', ' ', '-', '=', '~', '`', '?', ',', '.', ':', ';'
  };
  for (byte i = 0; i < strlen - 1; i++) answer += simbol[secureRandom(0, 81)];
  return md5(answer);
}

//================================================================начало==================================================================
 /*
   Вычисление разницы в давлении каждые два часа
 */

void pressureDinamic() {
  timerPressureDinamic = millis();
  deltaPress = pressure.data - oldPress;
  oldPress =  pressure.data;
  }

//================================================================конец====================================================================  
